@@ -151,7 +151,11 @@ class Node(BaseModel):
 
         url: str = None
 
-    # TODO make equality and hashing
+    def __eq__(self, other) -> bool:
+        return hash(self) == hash(other)
+
+    def __hash__(self) -> int:
+        return hash(f"{self.__class__.__name__}{self.json()}")
 
     @property
     def cache(self) -> CacheManager:
@@ -215,7 +219,7 @@ class Node(BaseModel):
         refresh: bool = False,
         strict: bool = False,
         use_stale: bool = False,
-    ) -> T.Optional[T.Union[NodeModel, T.List[NodeModel]]]:
+    ) -> T.Optional[T.Union[NodeModel, T.Set[NodeModel]]]:
         root_resolver = self.get_root_resolver()()
         if not resolver:
             resolver = root_resolver.edges.__fields__[name].type_()
@@ -265,6 +269,7 @@ class Node(BaseModel):
     def make_set_and_remove_lists(
         original_list: T.Optional[set], new_list: T.Optional[set]
     ) -> T.Tuple[set, set]:
+        # TODO should really ONLY be taking in SETS... fix later?
         # add all values from original list not in new list
         if original_list is None:
             original_list = set()
@@ -294,7 +299,7 @@ class Node(BaseModel):
             original_val = original_d.get(field_name, unset_key)
             if current_val != original_val:
                 # if list, if values have been removed, add to remove... if values have been added, keep
-                if {type(current_val), type(original_val)} & {list, set}:
+                if {type(current_val), type(original_val)} & {list, set, tuple}:
                     set_list, remove_list = self.make_set_and_remove_lists(
                         original_list=original_val, new_list=current_val
                     )
@@ -364,7 +369,7 @@ class Node(BaseModel):
         for field_name, node_or_nodes in rel_d_input.items():
             if node_or_nodes is None:
                 continue
-            if type(node_or_nodes) is list:
+            if type(node_or_nodes) in [set, list, tuple]:
                 val = {"id": n.id for n in node_or_nodes}
             else:
                 node_or_nodes: NodeModel
